@@ -2,9 +2,11 @@
 
 /* Endian conversion for reading and writing binary policies */
 
+#include <sepol/policydb/policydb.h>
+
 #include <byteswap.h>
 #include <endian.h>
-#include <sepol/policydb/policydb.h>
+#include <errno.h>
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #define cpu_to_le16(x) (x)
@@ -22,6 +24,9 @@
 #define le64_to_cpu(x) bswap_64(x)
 #endif
 
+#undef min
+#define min(a,b) (((a) < (b)) ? (a) : (b))
+
 /* Policy compatibility information. */
 struct policydb_compat_info {
 	unsigned int type;
